;;; -*- Mode: Lisp; Package: CB; Base: 10 -*-
#-Genera
(in-package :CB)

;;
;; Commands
;;

;;
;; cb-file
;;

(define-command com-show-info ()
  (let ((stream (get-frame-pane *application-frame* 'details)))
    (with-slots (class subtypes supertypes) *application-frame*
      (format stream "~&Class: ~S~%" class)
      (format stream "~&Subtypes: ~{~S~^, ~}~%" (loop repeat 5 for i in subtypes collect i))
      (format stream "~&Supertypes: ~{~S~&, ~}~%" (loop repeat 5 for i in supertypes collect i))
      )))

(define-command com-cb-exit ()
  (frame-exit *application-frame*))

;;
;; cb-class
;;

(define-command com-set-class ((cls 'cl:class))
  (with-slots (class subtypes supertypes) *application-frame*
    (setf class cls)
    (setf subtypes   (class-direct-subclasses   class))
    (setf supertypes (class-direct-superclasses class))
    (window-clear (get-frame-pane *application-frame* 'details))
    (window-clear (get-frame-pane *application-frame* 'subtypes))
    (window-clear (get-frame-pane *application-frame* 'supertypes))
    )
  )

(define-command com-reset-class ()
  (let ((item (find-class 'cl:standard-object)))
    (with-slots (class subtypes supertypes) *application-frame*
      (setf class item)
      (setf subtypes (class-direct-subclasses item))
      (setf supertypes (class-direct-superclasses item)))
    (window-clear (get-frame-pane *application-frame* 'subtypes))
    (window-clear (get-frame-pane *application-frame* 'details))
    (window-clear (get-frame-pane *application-frame* 'supertypes))
    ))

;;
;; cb-help
;;

(define-command com-help-info ()
  (let (#+LispWorks stream)
    (with-slots (class subtypes supertypes) *application-frame*
      (accepting-values (stream :own-window t :exit-boxes '((:exit "Done")
                                                            (:abort "Really Done"))
                                )
        (formatting-item-list (stream :row-wise nil :n-columns 1)
          (formatting-cell (stream :align-x :left) (format stream "Class:        ~S" class))
          (formatting-cell (stream :align-x :left) (format stream "Subclasses:   ~D" (length subtypes)))
          (formatting-cell (stream :align-x :left) (format stream "Superclasses: ~D" (length supertypes)))
          )))))

(define-command com-help-about ()
  ;; need to pop up window and display About message
  (let (#+LispWorks stream)
    (accepting-values (stream :own-window t :exit-boxes '((:exit  "Done")
                                                          (:abort "Really Done"))
                              )
      (formatting-item-list (stream :row-wise nil :n-columns 1)
        (formatting-cell (stream :align-x :center) (princ "This is the About message" stream))
        (formatting-cell (stream :align-x :center) (princ "for Class Browser"         stream))
        )
      )
    )
  )

;;
;; Command Tables
;;

(define-command-table cb-file
  :menu (;;
         ;; -------
         ;; Exit
         ("Show Info"    :command com-show-info)
         ("bfile"        :divider :line)
         ("Exit"         :command com-cb-exit)
         )
  )

(define-command-table cb-class
  :menu (;;
         ("Reset Class" :command com-reset-class)
         ("Set Class"   :command com-set-class)
         )
  )

(define-command-table cb-help
  :menu (;; Info
         ;; ----------
         ;; About
         ("Info"  :command com-help-info)
         ("bhelp" :divider :line)
         ("About" :command com-help-about)
         )
  )
