;;; -*- Mode: Lisp; Package: CB; Base: 10 -*-
#-Genera
(in-package :CB)

(defgeneric cb-top-level (frame &key))

(define-application-frame cb-browser ()
    ((class      :accessor cb-class)
     (subtypes   :accessor cb-subtypes)
     (supertypes :accessor cb-supertypes)
     )
 (:menu-bar t)
; (:top-level cb-top-level)
 (:command-definer nil)
 (:command-table (cb-browser
                  :inherit-from (cb-file cb-class cb-help)
                  :menu (("File"  :menu cb-file)
                         ("Class" :menu cb-class)
                         ("Help"  :menu cb-help)
                         )
                  )
                 )
 (:panes
  (subtypes   :application :incremental-redisplay t
              :end-of-line-action :allow :end-of-page-action :allow
              :display-function 'display-subtypes
              ;; display list of subtypes (cb-subtypes)
              )
  (details    :application :incremental-redisplay t
              :end-of-line-action :allow :end-of-page-action :allow
              :display-function 'display-details
              ;; display:
              ;;   Number of subtypes
              ;;   Number of supertypes
              ;;   list all slots, identifying which ones are local
              )
  (supertypes :application :incremental-redisplay t
              :end-of-line-action :allow :end-of-page-action :allow
              :display-function 'display-supertypes
              ;; display list of supertypes (cb-supertypes)
              )
  )
 (:layouts
  (main (vertically ()
          (1/3 supertypes)
          (:fill details)
          (1/3 subtypes))
        )
  )
 )

(defmethod cb-top-level ((frame cb-browser) &rest options)
;  (with-slots () frame
    (apply #'default-frame-top-level frame options)
;    )
  )

;; need other methods

(defun run-class-browser ()
  #-McCLIM
  (let ((fm (find-frame-manager)))
    (setf (frame-manager-palette fm) (make-palette (port fm))))
  (let ((frame (make-application-frame 'cb-browser
                                       :width 900
                                       :height 600)))
    (with-slots (class subtypes supertypes) frame
      (setf class (find-class 'cl:standard-object))
      (setf subtypes (class-direct-subclasses class))
      (setf supertypes (class-direct-superclasses class))
      ;; Print Info
      (format t "~&Class: ~S~%" class)
      (format t "~&Subclasses: ~{~S~^, ~}~%" (loop repeat 5 for i in subtypes collect (class-name i)))
      (format t "~&Superclasses: ~{~S~^, ~}~%" (loop repeat 5 for i in supertypes collect (class-name i)))
      )
;    (redisplay-frame-panes frame :force-p t)
    (run-frame-top-level frame)))

(defun display-details (frame pane &key)
  (with-slots (class) frame
    (princ "Class: " pane) (present class (presentation-type-of class) :stream pane) (terpri pane)
    (let ((all (class-slots class))
          (local (loop for slot in (class-direct-slots class)
                   collect (slot-definition-name slot)))
          )
      (formatting-item-list (pane)
        (loop for slot in all
          as local-p = (member (slot-definition-name slot) local)
          do (formatting-cell (pane) (format pane "~:[local ~;Inherit ~]~S~%" local-p slot)))))))

(defun display-supertypes (frame pane &key)
  (with-slots (supertypes) frame
    (formatting-item-list (pane)
      (loop for super in supertypes
        do (formatting-cell (pane)
             (present super (presentation-type-of super) :stream pane))
        ))))

(defun display-subtypes (frame pane &key)
  (with-slots (subtypes) frame
    (formatting-item-list (pane)
      (loop for sub in subtypes
        do (formatting-cell (pane) (present sub (presentation-type-of sub) :stream pane))
      ))))
